rm(list = ls())
##################################################################
## First,  "cmp" package from github needs to installed with the following command.
## devtools::install_github("SuneelChatla/cmp")
##################################################################
### Loading the necessary libraries
library(cmp)
library(partykit)
library(devtools)
library(RCurl)

###################################################################
### The source code for the CMPMOB and CMPBoost functions are available on https://github.com/SuneelChatla/CMPTree
### Need to source the functions from github. Please note that its not a package.
###################################################################

source_url("https://raw.githubusercontent.com/SuneelChatla/CMPTree/master/mobvc.R")
source_url("https://raw.githubusercontent.com/SuneelChatla/CMPTree/master/bt-cmp.R")
source_url("https://raw.githubusercontent.com/SuneelChatla/CMPTree/master/glm-cmp.R")
source_url("https://raw.githubusercontent.com/SuneelChatla/CMPTree/master/mob_pvalue.R")


############################################################################
############## Simulation Study in Section 3.3   ###########################
############################################################################

library(microbenchmark)

### Function for the example data generation for CMPMOB Tree

data.sim <- function(n, sd) {
  set.seed(sd)
  # covariates
  sdata <- data.frame("x1" = runif(n, 0, 1))
  sdata$x2 <- runif(n, 0, 1)
  sdata$x3 <- runif(n, 0, 1)
  sdata$w1 <- runif(n, 0, 1)
  sdata$w2 <- runif(n, 0, 1)
  # sdata$w3 <- runif(n, 0, 1)
  # moderators
  sdata$z1 <- (runif(n, 0, 1))
  sdata$z2 <- runif(n, 0, 1)
  sdata$z3 <- runif(n, 0, 1)
  sdata$z4 <- runif(n, 0, 1)
  # nonparametric function
  f1 <- function(x) sin(2 * pi * x)
  f2 <- function(x) cos(2 * pi * x)
  ## linear predictor for lambda
  sdata$eta1 <- 2 + (sdata$z1 > 0.65) * (2 * sdata$x1) + (sdata$z1 <= 0.65) * sdata$x2 + 2 * f1(sdata$x3)^2
  lambda <- exp(sdata$eta1)
  sdata$eta2 <- 0.25 + 0.5 * (sdata$z3 > 0.5) * sdata$w1 + 0.5 * f2(sdata$w2)^2 #* ((sdata$z1>0.5)) for ex2
  nu <- exp(sdata$eta2)
  s <- rep(0, n)
  # simulating y from CMP
  y <- .C("cmpsim_all", lambda = as.double(lambda), nu = as.double(nu), n = as.integer(n), y = as.double(s))$y
  sdata$y <- y
  #
  return(sdata)
}


############################################################################
################### Input model formulation ###############################
### formula: list of formulas for both lambda and nu models that will be estimated in the MOB Tree
### fformula: list of formulas for both lambda and nu models that involve only fixed effects. These terms are not part of tree estimation.
### vcformula: formula for moderator variables. For CMPMOB tree, both lambda and nu models are constrained to use the same set of moderators.
###########################################################################

formula <- list(as.formula(y ~ x1 + x2), as.formula(y ~ w1)) #
fformula <- list(as.formula(~ s(x3)), as.formula(~ s(w2)))
vcformula <- as.formula(~ z1 + z2 + z3 + z4)

#########################################################################
############## Repeated simulations experiment ##########################

ssize <- c(500, 1000, 2000, 5000)
rep <- 50
#
split1 <- matrix(NA, nrow = rep, ncol = length(ssize))
split2 <- matrix(NA, nrow = rep, ncol = length(ssize))
split3 <- matrix(NA, nrow = rep, ncol = length(ssize))
#
gdev1 <- matrix(NA, nrow = rep, ncol = length(ssize))
gdev2 <- matrix(NA, nrow = rep, ncol = length(ssize))
gdev3 <- matrix(NA, nrow = rep, ncol = length(ssize))
#
ldev1 <- matrix(NA, nrow = rep, ncol = length(ssize))
ldev2 <- matrix(NA, nrow = rep, ncol = length(ssize))
ldev3 <- matrix(NA, nrow = rep, ncol = length(ssize))
#
nterm1 <- matrix(NA, nrow = rep, ncol = length(ssize))
nterm2 <- matrix(NA, nrow = rep, ncol = length(ssize))
nterm3 <- matrix(NA, nrow = rep, ncol = length(ssize))
#
medtime1 <- matrix(NA, nrow = rep, ncol = length(ssize))
medtime2 <- matrix(NA, nrow = rep, ncol = length(ssize))
medtime3 <- matrix(NA, nrow = rep, ncol = length(ssize))

###
for (i in 1:length(ssize)) {
  for (j in 1:rep) {
    # simulate data
    data <- data.sim(ssize[i], sd = 123 + j)

    tp1 <- microbenchmark( ### vc model with exhaustive search
      cmp.mob1 <- cmpmob(formula, fformula, vcformula, data = data, family = cmp, sflag = TRUE, control.mob = mob_control()),
      times = 1L, unit = "s"
    )
    ### vc model with change point method- exact stat
    tp2 <- microbenchmark(cmp.mob2 <- cmpmob(formula, fformula, vcformula, data = data, family = cmp, sflag = FALSE, cperc = FALSE, control.mob = mob_control()), times = 1L, unit = "s")
    ### vc model with change point method- 10perc stat
    tp3 <- microbenchmark(cmp.mob3 <- cmpmob(formula, fformula, vcformula, data = data, family = cmp, sflag = FALSE, cperc = TRUE, control.mob = mob_control()), times = 1L, unit = "s")

    #### Extracting required info from the fit
    ## global -2logLik
    gdev1[j, i] <- cmp.mob1$glob$deviance
    ## Local -2logLik
    # for exhaust search
    node1 <- nodeids(cmp.mob1$tree, terminal = TRUE)
    ldev1[j, i] <- sum(unlist(nodeapply(cmp.mob1$tree, ids = node1, FUN = function(n) info_node(n)$deviance)))
    ## split
    a1 <- unlist(nodeapply(cmp.mob1$tree, ids = nodeids(cmp.mob1$tree), FUN = function(n) split_node(n)$breaks))
    a11 <- unlist(nodeapply(cmp.mob1$tree, ids = nodeids(cmp.mob1$tree), FUN = function(n) split_node(n)$varid))
    nterm1[j, i] <- if (is.null(a11)) NA else paste(format(a11), collapse = ",")
    split1[j, i] <- if (is.null(a1)) NA else paste(format(a1, digits = 3), collapse = ",")

    # for exact set of change point
    gdev2[j, i] <- cmp.mob2$glob$deviance
    node2 <- nodeids(cmp.mob2$tree, terminal = TRUE)
    # nterm2[j,i] <- length(node2)
    ldev2[j, i] <- sum(unlist(nodeapply(cmp.mob2$tree, ids = node2, FUN = function(n) info_node(n)$deviance)))
    ## split
    a2 <- unlist(nodeapply(cmp.mob2$tree, ids = nodeids(cmp.mob2$tree), FUN = function(n) split_node(n)$breaks))
    a21 <- unlist(nodeapply(cmp.mob2$tree, ids = nodeids(cmp.mob2$tree), FUN = function(n) split_node(n)$varid))
    nterm2[j, i] <- if (is.null(a21)) NA else paste(format(a21), collapse = ",")
    split2[j, i] <- if (is.null(a2)) NA else paste(format(a2, digits = 4), collapse = ",")

    ## for 10% of change point
    gdev3[j, i] <- cmp.mob3$glob$deviance
    node3 <- nodeids(cmp.mob3$tree, terminal = TRUE)
    # nterm3[j,i] <- length(node3)
    ldev3[j, i] <- sum(unlist(nodeapply(cmp.mob3$tree, ids = node3, FUN = function(n) info_node(n)$deviance)))
    ## split
    a3 <- unlist(nodeapply(cmp.mob3$tree, ids = nodeids(cmp.mob3$tree), FUN = function(n) split_node(n)$breaks))
    a31 <- unlist(nodeapply(cmp.mob3$tree, ids = nodeids(cmp.mob3$tree), FUN = function(n) split_node(n)$varid))
    nterm3[j, i] <- if (is.null(a31)) NA else paste(format(a31), collapse = ",")
    split3[j, i] <- if (is.null(a3)) NA else paste(format(a3, digits = 4), collapse = ",")
    ## Computational times
    # ntp <- as.matrix(tp)
    medtime1[j, i] <- tp1[, 2] / 1e+9
    medtime2[j, i] <- tp2[, 2] / 1e+9
    medtime3[j, i] <- tp3[, 2] / 1e+9
    cat("i:", i, ",", "j:", j)
  }
}

### Storing the results
store1 <- cbind(rbind(gdev1, gdev2, gdev3), rbind(ldev1, ldev2, ldev3), rbind(nterm1, nterm2, nterm3), rbind(split1, split2, split3), rbind(medtime1, medtime2, medtime3))
write.csv(store1, "sim22-50.csv")

##############################################################
################## deviance plot :Figure A2 ##################

pdf("devplot-sim2.pdf", 11, 9)
par(mfrow = c(2, 2))
plot(1:50, gdev1[, 1], type = "l", lwd = 2, xlab = "repetition", ylab = "-2l", ylim = c(2000, 2900), main = "Sample size=500", cex.axis = 1.5, cex.lab = 1.5)
points(1:50, ldev1[, 1], type = "l", lwd = 2, lty = 1, col = "darkgray")
points(1:50, ldev2[, 1], type = "l", lwd = 2, lty = 2)
points(1:50, ldev3[, 1], type = "l", lwd = 2, lty = 3)
#
plot(1:50, gdev1[, 2], type = "l", lwd = 2, xlab = "repetition", ylab = "-2l", ylim = c(4000, 6000), main = "Sample size=1000", cex.axis = 1.5, cex.lab = 1.5)
points(1:50, ldev1[, 2], type = "l", lwd = 2, lty = 1, col = "darkgray")
points(1:50, ldev2[, 2], type = "l", lwd = 2, lty = 2)
points(1:50, ldev3[, 2], type = "l", lwd = 2, lty = 3)
#
plot(1:50, gdev1[, 3], type = "l", lwd = 2, xlab = "repetition", ylab = "-2l", ylim = c(9000, 12000), main = "Sample size=2000", cex.axis = 1.5, cex.lab = 1.5)
points(1:50, ldev1[, 3], type = "l", lwd = 2, lty = 1, col = "darkgray")
points(1:50, ldev2[, 3], type = "l", lwd = 2, lty = 2)
points(1:50, ldev3[, 3], type = "l", lwd = 2, lty = 3)
#
plot(1:50, gdev1[, 4], type = "l", lwd = 2, xlab = "repetition", ylab = "-2l", ylim = c(23000, 29000), main = "Sample size=5000", cex.axis = 1.5, cex.lab = 1.5)
points(1:50, ldev1[, 4], type = "l", lwd = 2, lty = 1, col = "darkgray")
points(1:50, ldev2[, 4], type = "l", lwd = 2, lty = 2)
points(1:50, ldev3[, 4], type = "l", lwd = 2, lty = 3)
dev.off()


###############################################################
############## Fixed effects plot:  Figure 1 ##################

pdf("fixef-s1.pdf", 7, 4)
plot.gam.cmp(cmp.mob1$fixed, pages = 1)
dev.off()

###############################################################
############### CMPMOB Tree plot: Figure 1 ####################

pdf("tree-s2b.pdf", 9, 7)
plot(cmp.mob1$tree)
dev.off()

### summary of sim1 results : Table 1
### summary statistics for splits
apply(split1, 2, function(x) mean(as.numeric(x), na.rm = TRUE))
apply(split1, 2, function(x) sd(as.numeric(x), na.rm = TRUE))

apply(split2, 2, function(x) mean(as.numeric(x), na.rm = TRUE))
apply(split2, 2, function(x) sd(as.numeric(x), na.rm = TRUE))

apply(split3, 2, function(x) mean(as.numeric(x), na.rm = TRUE))
apply(split3, 2, function(x) sd(as.numeric(x), na.rm = TRUE))


### Function to extract multiple splits
mspl <- function(x, y, nc = 3) {
  sl <- length(x)
  smat <- matrix(NA, sl, nc)
  tmat <- matrix(NA, sl, nc)
  for (i in 1:sl)
  {
    ts <- as.numeric(unlist(strsplit(x[i], ",")))
    tt <- as.numeric(unlist(strsplit(y[i], ",")))
    smat[i, 1:length(ts)] <- ts
    tmat[i, 1:length(tt)] <- tt
  }
  rm <- vector("list", nc)
  rs <- vector("list", nc)
  rt <- vector("list", nc)
  for (i in 1:nc)
  {
    tu <- na.omit(unique(tmat[, i]))
    tmean <- rep(NA, length(tu))
    tsd <- rep(NA, length(tu))
    for (j in 1:length(tu))
    {
      tts.ind <- tmat[, i] == tu[j]
      tmean[j] <- mean(smat[tts.ind, i], na.rm = TRUE)
      tsd[j] <- sd(smat[tts.ind, i], na.rm = TRUE)
    }
    rm[[i]] <- tmean
    rs[[i]] <- tsd
    rt[[i]] <- tu
  }

  return(list(m = rm, s = rs, t = rt))
}

# Exhaustive search summary
mspl(split1[, 1], nterm1[, 1])
mspl(split1[, 2], nterm1[, 2],nc=4)
mspl(split1[, 3], nterm1[, 3], nc = 4)
mspl(split1[, 4], nterm1[, 4])
# Exact change point summary
mspl(split2[, 1], nterm2[, 1])
mspl(split2[, 2], nterm2[, 2])
mspl(split2[, 3], nterm2[, 3])
mspl(split2[, 4], nterm2[, 4])
# 10-percent change point summary
mspl(split3[, 1], nterm3[, 1])
mspl(split3[, 2], nterm3[, 2],nc=4)
mspl(split3[, 3], nterm3[, 3], nc = 4)
mspl(split3[, 4], nterm3[, 4])
#

##############################################################
####### Deviance/-2l Summary for Table 1 #####################

apply(gdev1, 2, mean)
apply(gdev1, 2, sd)

apply(gdev2, 2, mean)
apply(gdev2, 2, sd)

apply(gdev3, 2, mean)
apply(gdev3, 2, sd)

#
apply(ldev1, 2, mean)
apply(ldev1, 2, sd)

apply(ldev2, 2, mean)
apply(ldev2, 2, sd)

apply(ldev3, 2, mean)
apply(ldev3, 2, sd)


#
apply(nterm1, 2, function(x) mean(as.numeric(x), na.rm = TRUE))
apply(nterm1, 2, function(x) sd(as.numeric(x), na.rm = TRUE))

apply(nterm2, 2, function(x) mean(as.numeric(x), na.rm = TRUE))
apply(nterm2, 2, function(x) sd(as.numeric(x), na.rm = TRUE))

apply(nterm3, 2, function(x) mean(as.numeric(x), na.rm = TRUE))
apply(nterm3, 2, function(x) sd(as.numeric(x), na.rm = TRUE))




###
par(mfrow = c(1, 1))
# function for error bars
error.bar <- function(x, y, upper, lower = upper, length = 0.1, ...) {
  if (length(x) != length(y) | length(y) != length(lower) | length(lower) != length(upper)) {
    stop("vectors must be same length")
  }
  arrows(x, y + upper, x, y - lower, angle = 90, code = 3, length = length, ...)
}

# function for standard error of the mean
sem <- function(x) {
  sd(x) / sqrt(length(x))
}

##################################################################
################  Figure 3 : Computation Times ###################

pdf("mob-comput.pdf", 7, 5)
c1m <- apply(medtime1, 2, mean)
c1s <- apply(medtime1, 2, sem)

plot(ssize, c1m, xlab = "Sample Size", ylab = "Time (in Seconds)", type = "l", lwd = 2, cex.axis = 1.5, cex.lab = 1.5)
error.bar(ssize, c1m, c1s)
#
c2m <- apply(medtime2, 2, mean)
c2s <- apply(medtime2, 2, sem)
points(ssize, c2m, type = "l", lty = 2, lwd = 2)
error.bar(ssize, c2m, c2s)
#
c3m <- apply(medtime3, 2, mean)
c3s <- apply(medtime3, 2, sem)
points(ssize, c3m, type = "l", lty = 4, lwd = 2)
error.bar(ssize, c3m, c3s)
#
legend(1000, 350, legend = c("Exhaustive search", "Exact change points", "Ten percent change points"), lty = c(1, 2, 4))
dev.off()
#####


##############################################################
######## plotting ES-CP Illustration: Figure 2 #######################

cpd <- read.csv("cp-plot.csv", header = FALSE, sep = "")
colnames(cpd) <- c("z", "Ds")
head(cpd)
esd <- read.csv("es-plot.csv", header = TRUE)
head(esd)

pdf("ES-CP-Stat.pdf", 11, 7)
par(mfrow = c(2, 3))
#
plot(esd[, 1], esd[, 2], xlab = "z1", ylab = "-2l", type = "l", lwd = 2, main = "ES", cex.axis = 1.2, cex.lab = 1.5,cex.main=1.75)
abline(v = 0.65, lty = 2, lwd = 1)

#
t1 <- expression(paste("CP: Score function -  ", beta[0]))
plot(cpd[1:4920, 1], cpd[1:4920, 2], xlab = "z1", ylab = "GLR Statistic", type = "l", lwd = 2, main = t1, cex.axis = 1.2, cex.lab = 1.5,cex.main=1.75)
abline(v = 0.65, lty = 2, lwd = 1)
#
t2 <- expression(paste("CP: Score function -  ", beta[1]))
plot(cpd[4921:9840, 1], cpd[4921:9840, 2], xlab = "z1", ylab = "GLR Statistic", type = "l", lwd = 2, main = t2, cex.axis = 1.25, cex.lab = 1.5,cex.main=1.75)
abline(v = 0.65, lty = 2, lwd = 1)
#
t3 <- expression(paste("CP: Score function -  ", beta[2]))
plot(cpd[9841:14760, 1], cpd[9841:14760, 2], xlab = "z1 ", ylab = "GLR Statistic", type = "l", lwd = 2, main = t3, cex.axis = 1.25, cex.lab = 1.5,cex.main=1.75)
abline(v = 0.65, lty = 2, lwd = 1)
#
t4 <- expression(paste("CP: Score function -  ", gamma[0]))
plot(cpd[14761:19680, 1], cpd[14761:19680, 2], xlab = "z1", ylab = "GLR Statistic", type = "l", lwd = 2, main = t4, cex.axis = 1.25, cex.lab = 1.5,cex.main=1.75)
abline(v = 0.65, lty = 2, lwd = 1)
#
t5 <- expression(paste("CP: Score function -  ", gamma[1]))
plot(cpd[19681:24650, 1], cpd[19681:24650, 2], xlab = "z1 ", ylab = "GLR Statistic", type = "l", lwd = 2, main = t5, cex.axis = 1.25, cex.lab = 1.5,cex.main=1.75)
abline(v = 0.65, lty = 2, lwd = 1)

dev.off()





#################################################################################
###################### Boosting simulation: Section 4.3 #########################
################################################################################
rm(list = ls())


## Function for example data generation of CMPBoost

data.sim.boost <- function(n, sd) {
  set.seed(sd)
  # covariates
  sdata <- data.frame("x1" = runif(n, 0, 1))
  # sdata$x2 <- runif(n,0,1)
  # sdata$x3 <- runif(n, 0, 1)
  sdata$w1 <- runif(n, 0, 1)
  # sdata$w2 <- runif(n, 0, 1)
  # sdata$w3 <- runif(n, 0, 1)
  # moderators
  sdata$z1 <- (runif(n, 0, 1))
  sdata$z2 <- runif(n, 0, 1)
  sdata$z3 <- runif(n, 0, 1)
  sdata$z4 <- runif(n, 0, 1)
  sdata$z5 <- (runif(n, 0, 1))
  sdata$z6 <- runif(n, 0, 1)
  sdata$z7 <- runif(n, 0, 1)
  sdata$z8 <- runif(n, 0, 1)
  sdata$z9 <- runif(n, 0, 1)
  sdata$z10 <- runif(n, 0, 1)

  # nonparametric function
  f1 <- function(x, y) 2 * sin(2 * pi * x)^2 + exp(y - 1)
  f2 <- function(x, y) 2 * cos(2 * pi * x)^2 + y * (1 - y)
  g1 <- function(x) sin(2 * pi * x)^2
  g2 <- function(x) 0.5 * cos(2 * pi * x)^2
  ## linear predictor for lambda
  sdata$f1 <- f1(sdata$z1, sdata$z2) # scale(f1(sdata$z1,sdata$z2),scale=FALSE);  #f1(sdata$z1,sdata$z2)  #
  sdata$f2 <- f2(sdata$z1, sdata$z2) # scale(f2(sdata$z1,sdata$z2),scale=FALSE); #f2(sdata$z1,sdata$z2)  #
  sdata$g1 <- g1(sdata$z5) # scale(g1(sdata$z1),scale=FALSE)
  sdata$g2 <- g2(sdata$z6) # scale(g2(sdata$z2),scale=FALSE)
  sdata$eta1 <- sdata$f1 + sdata$x1 * sdata$f2
  lambda <- exp(sdata$eta1)
  sdata$eta2 <- sdata$g1 + sdata$w1 * sdata$g2
  # sdata$eta2 <- 0.25 + 0.25*sdata$w1#sdata$g1+sdata$w1*sdata$g2  #-0.25 + 0.45*sdata$w1
  nu <- exp(sdata$eta2) # rep(1.5,n) #exp(sdata$eta2)
  s <- rep(0, n)
  # simulating y from CMP
  y <- .C("cmpsim_all", lambda = as.double(lambda), nu = as.double(nu), n = as.integer(n), y = as.double(s))$y
  sdata$y <- y
  #
  return(sdata)
}

## training and test data
index.train <- (1:n) %in% sample(1:n, round(n * 0.6))
split.nd <- "1"
Leaf <- rep("1", n)



############ Model formulations for CMPBoost ############################################
#########################################################################################

formula <- list(as.formula(y ~ x1), as.formula(y ~ w1)) # as.formula(y~x1) #
fformula <- NULL
vcformula <- list(as.formula(~ z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + z9 + z10 - 1), as.formula(~ z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + z9 + z10 - 1))
# formula <-list(as.formula(y~x1),as.formula(y~w1)) #as.formula(y~x1) #
# fformula <- NULL
# vcformula <- as.formula(~z1+z2+z3+z4+z5+z6+z7+z8+z9+z10-1) #list(as.formula(~z1+z2+z3+z4+z5+z6+z7+z8+z9+z10-1),as.formula(~z1+z2+z3+z4+z5+z6+z7+z8+z9+z10-1))


##################### Tuning parameter M #############

grid.M <- c(5, 10, 15, 20, 25, 30)

# grid.M1 <- c(45,50,55,60,65,70,75,80)
tdev <- rep(0, length(grid.M))
terr <- rep(0, length(grid.M))
tmin <- rep(0, length(grid.M))
niter <- rep(0, length(grid.M))
tdev1 <- tdev
terr1 <- terr
tmin1 <- tmin
niter1 <- niter
for (i in 1:length(grid.M))
{
  tres <- Boost.Tree.basic(formula = formula, fformula = fformula, vcformula = vcformula, data = sdata[index.train, ], eta = 0.1, mini.size = 20, M = grid.M[i], nBoost = 500)
  tdev[i] <- tres$deviance[tres$iter]
  terr[i] <- tres$error.sq[which.min(tres$error.sq[1:tres$iter])]
  tmin[i] <- which.min(tres$error.sq[1:tres$iter])
  niter[i] <- tres$iter
  #
  tres1 <- Boost.Tree.basic(formula = formula, fformula = fformula, vcformula = vcformula, data = sdata[!index.train, ], eta = 0.1, mini.size = 20, M = grid.M[i], nBoost = tres$iter)
  tdev1[i] <- tres1$deviance[tres1$iter]
  terr1[i] <- tres1$error.sq[which.min(tres1$error.sq[1:tres1$iter])]
  tmin1[i] <- which.min(tres1$error.sq[1:tres1$iter])
  niter1[i] <- tres1$iter
} ## 0.05 for nu function



############## Training vs Testing plot: Figure 4 ##############################
################################################################################

pdf("gb-train2-m-ex2.pdf", 7, 4)
#
par(mfrow = c(1, 2))
plot(grid.M, tdev, xlab = "M", ylab = "-2l", pch = 1, type = "l", lwd = 2, cex.axis = 1.5, cex.lab = 1.5, ylim = c(1200, 2200))
points(grid.M, tdev1, type = "l", lwd = 2, lty = 2)
#
plot(grid.M, terr, xlab = "M", ylab = "Prediction Error", pch = 1, type = "l", lwd = 2, cex.axis = 1.5, cex.lab = 1.5, ylim = c(0, 750))
points(grid.M, terr1, type = "l", lwd = 2, lty = 2)
dev.off()



#########################################################################################
### partial dependecy plot creation across 20 simulations: Figure 5 and 6
# #######################################################################################

formula <- list(as.formula(y ~ x1), as.formula(y ~ w1)) # as.formula(y~x1) #
fformula <- NULL
vcformula <- list(as.formula(~ z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + z9 + z10 - 1), as.formula(~ z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + z9 + z10 - 1))

##
nsim <- 20
bt.res <- vector(mode = "list", length = nsim)
Emob.res <- bt.res
Cmob.res <- bt.res
res <- bt.res
emres <- bt.res
cmres <- bt.res
#
for (i in 1:nsim)
{
  sdata <- data.sim.boost(n = 500, sd = 10 + i)

  res[[i]] <- Boost.Tree(formula = formula, fformula = fformula, vcformula = vcformula, data = sdata, eta = 0.1, mini.size = 20, M = 15, nBoost = 500)
  bt.res[[i]] <- Partial.Boost(res[[i]], name.vars = c("z1", "z2", "z5", "z6"))
  # part.res2 <- Partial.Boost(res,c("z3","z4"))
  emres[[i]] <- cmpmob(formula, fformula, vcformula, data = sdata, family = cmp, sflag = TRUE)
  Emob.res[[i]] <- Partial.mob(emres[[i]], name.vars = c("z1", "z2", "z5", "z6"))
  ### vc model with change point method
  cmres[[i]] <- cmpmob(formula, fformula, vcformula, data = sdata, family = cmp, sflag = FALSE)
  Cmob.res[[i]] <- Partial.mob(cmres[[i]], name.vars = c("z1", "z2", "z5", "z6"))
}

###




###
### plotting partial residual plot
##
f1 <- function(x, y) 2 * sin(2 * pi * x)^2 + exp(y - 1)
f2 <- function(x, y) 2 * cos(2 * pi * x)^2 + y * (1 - y)
g1 <- function(x) sin(2 * pi * x)^2
g2 <- function(x) 0.5 * cos(2 * pi * x)^2

#### Storing the partial plots

pdf("partial-sim2-2lamfunc.pdf", 9, 4)
par(mfrow = c(2, 3))
#
## plot 1

par(mar = c(5, 5, 0.5, 0.5))
for (i in 1:nsim)
{
  bdat <- bt.res[[i]]
  if (i == 1) {
    plot(bdat$ulevel.lam[, 1], f1(bdat$ulevel.lam[, 1], mean(bdat$ulevel.lam[, 2])), type = "l", xlab = expression(z[1]), ylim = c(-1, 3), col = "grey", ylab = expression(beta[0](z[1])), cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(bdat$ulevel.lam[, 1], f1(bdat$ulevel.lam[, 1], mean(bdat$ulevel.lam[, 2])), type = "l", col = "grey")
  }

  points(bdat$ulevel.lam[, 1], bdat$coefs.lam[[1]][, 1], pch = ".")
}

#
# par(mar = c(2, 2, 0.5, 0.5))
## plot 2
for (i in 1:nsim)
{
  emdat <- Emob.res[[i]]
  if (i == 1) {
    plot(emdat$ulevel.lam[, 1], f1(emdat$ulevel.lam[, 1], mean(emdat$ulevel.lam[, 2])), type = "l", ylab = "", xlab = expression(z[1]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(emdat$ulevel.lam[, 1], f1(emdat$ulevel.lam[, 1], mean(emdat$ulevel.lam[, 2])), type = "l", col = "grey")
  }
  points(emdat$ulevel.lam[, 1], emdat$coefs.lam[[1]][, 1], pch = ".")
}

## plot 3
for (i in 1:nsim)
{
  cmdat <- Cmob.res[[i]]
  if (i == 1) {
    plot(cmdat$ulevel.lam[, 1], f1(cmdat$ulevel.lam[, 1], mean(cmdat$ulevel.lam[, 2])), type = "l", ylab = "", xlab = expression(z[1]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(cmdat$ulevel.lam[, 1], f1(cmdat$ulevel.lam[, 1], mean(cmdat$ulevel.lam[, 2])), type = "l", col = "grey")
  }
  points(cmdat$ulevel.lam[, 1], cmdat$coefs.lam[[1]][, 1], pch = ".")
}


### second function

# par(mfrow=c(1,3))
#

## plot 1


for (i in 1:nsim)
{
  bdat <- bt.res[[i]]
  if (i == 1) {
    plot(bdat$ulevel.lam[, 1], f2(bdat$ulevel.lam[, 1], mean(bdat$ulevel.lam[, 2])), type = "l", xlab = expression(z[1]), ylim = c(-1, 3), col = "grey", ylab = expression(beta[1](z[1])), cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(bdat$ulevel.lam[, 1], f2(bdat$ulevel.lam[, 1], mean(bdat$ulevel.lam[, 2])), type = "l", col = "grey")
  }

  points(bdat$ulevel.lam[, 1], bdat$coefs.lam[[1]][, 2], pch = ".")
}

## plot 2
for (i in 1:nsim)
{
  emdat <- Emob.res[[i]]
  if (i == 1) {
    plot(emdat$ulevel.lam[, 1], f2(emdat$ulevel.lam[, 1], mean(emdat$ulevel.lam[, 2])), type = "l", ylab = "", xlab = expression(z[1]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(emdat$ulevel.lam[, 1], f2(emdat$ulevel.lam[, 1], mean(emdat$ulevel.lam[, 2])), type = "l", col = "grey")
  }
  points(emdat$ulevel.lam[, 1], emdat$coefs.lam[[1]][, 2], pch = ".")
}

## plot 3
for (i in 1:nsim)
{
  cmdat <- Cmob.res[[i]]
  if (i == 1) {
    plot(cmdat$ulevel.lam[, 1], f2(cmdat$ulevel.lam[, 1], mean(cmdat$ulevel.lam[, 2])), type = "l", ylab = "", xlab = expression(z[1]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(cmdat$ulevel.lam[, 1], f2(cmdat$ulevel.lam[, 1], mean(cmdat$ulevel.lam[, 2])), type = "l", col = "grey")
  }
  points(cmdat$ulevel.lam[, 1], cmdat$coefs.lam[[1]][, 2], pch = ".")
}

dev.off()


######################################################################################
################# for nu functions ###################################################


pdf("partial-sim2-2nufunc.pdf", 9, 4)
par(mfrow = c(2, 3))
#
## plot 1

par(mar = c(5, 5, 0.5, 0.5))
for (i in 1:nsim)
{
  bdat <- bt.res[[i]]
  if (i == 1) {
    plot(bdat$ulevel.nu[, 1], g1(bdat$ulevel.nu[, 1]), type = "l", xlab = expression(z[5]), ylim = c(-1, 3), col = "grey", ylab = expression(gamma[0](z[5])), cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(bdat$ulevel.nu[, 1], g1(bdat$ulevel.nu[, 1]), type = "l", col = "grey")
  }
  
  points(bdat$ulevel.nu[, 1], bdat$coefs.nu[[3]][, 1], pch = ".")
}

#
# par(mar = c(2, 2, 0.5, 0.5))
## plot 2
for (i in 1:nsim)
{
  emdat <- Emob.res[[i]]
  if (i == 1) {
    plot(emdat$ulevel.lam[, 3], g1(emdat$ulevel.lam[, 3]), type = "l", ylab = "", xlab = expression(z[5]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(emdat$ulevel.lam[, 3], g1(emdat$ulevel.lam[, 3]), type = "l", col = "grey")
  }
  points(emdat$ulevel.lam[, 3], emdat$coefs.nu[[3]][, 1], pch = ".")
}

## plot 3
for (i in 1:nsim)
{
  cmdat <- Cmob.res[[i]]
  if (i == 1) {
    plot(cmdat$ulevel.lam[, 3], g1(cmdat$ulevel.lam[,3]), type = "l", ylab = "", xlab = expression(z[5]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(cmdat$ulevel.lam[, 3], g1(cmdat$ulevel.lam[, 3]), type = "l", col = "grey")
  }
  points(cmdat$ulevel.lam[, 3], cmdat$coefs.nu[[3]][, 1], pch = ".")
}


### second function

# par(mfrow=c(1,3))
#

## plot 1


for (i in 1:nsim)
{
  bdat <- bt.res[[i]]
  if (i == 1) {
    plot(bdat$ulevel.lam[, 4], g2(bdat$ulevel.lam[, 4]), type = "l", xlab = expression(z[6]), ylim = c(-1, 3), col = "grey", ylab = expression(gamma[1](z[6])), cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(bdat$ulevel.lam[, 4], g2(bdat$ulevel.lam[, 4]), type = "l", col = "grey")
  }
  
  points(bdat$ulevel.lam[, 4], bdat$coefs.nu[[4]][, 2], pch = ".")
}

## plot 2
for (i in 1:nsim)
{
  emdat <- Emob.res[[i]]
  if (i == 1) {
    plot(emdat$ulevel.lam[, 4], g2(emdat$ulevel.lam[, 4]), type = "l", ylab = "", xlab = expression(z[6]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(emdat$ulevel.lam[, 4], g2(emdat$ulevel.lam[, 4]), type = "l", col = "grey")
  }
  points(emdat$ulevel.lam[, 4], emdat$coefs.nu[[4]][, 2], pch = ".")
}

## plot 3
for (i in 1:nsim)
{
  cmdat <- Cmob.res[[i]]
  if (i == 1) {
    plot(cmdat$ulevel.lam[, 4], g2(cmdat$ulevel.lam[, 4]), type = "l", ylab = "", xlab = expression(z[6]), ylim = c(-1, 3), col = "grey", cex.axis = 1.5, cex.lab = 1.75)
  } else {
    points(cmdat$ulevel.lam[, 4], g2(cmdat$ulevel.lam[, 4]), type = "l", col = "grey")
  }
  points(cmdat$ulevel.lam[, 4], cmdat$coefs.nu[[4]][, 2], pch = ".")
}

dev.off()




######################################################################
############ variable importance plot: Figure 7 ######################


pdf("VarImp-sim2-ex2.pdf", 9, 4)
par(mfrow = c(1, 2))
par(mar = c(2, 2, 0.5, 0.5))
znames <- c(expression(z[1]), expression(z[2]), expression(z[3]), expression(z[4]), expression(z[5]), expression(z[6]), expression(z[7]), expression(z[8]), expression(z[9]), expression(z[10]))

barplot(res[[2]]$variable.imp.lam, names.arg = znames, cex.names = 1.5)
barplot(res[[18]]$variable.imp.lam, names.arg = znames, cex.names = 1.5)

dev.off()




################################################################################
###########  Bikesharing Example: Section 5 ####################################
################################################################################
rm(list = ls())
## reading bike-sharing data

hr <- read.csv("/home/sbchatla/Dropbox/CMP-IRLS and GAM/Bike-Sharing-Dataset/hour.csv", header = T)

head(hr)

# variable coding
hr$season <- with(hr, factor(season,
  levels = c(1, 2, 3, 4),
  labels = c("springer", "summer", "fall", "winter")
))
hr$yr <- with(hr, factor(yr, levels = c(0, 1), labels = c("2011", "2012")))
hr$mnth <- with(hr, factor(mnth,
  levels = c(1:12),
  labels = c(
    "jan", "feb", "mar", "apr", "may", "jun",
    "jul", "aug", "sep", "oct", "nov", "dec"
  )
))
hr$holiday <- with(hr, factor(holiday,
  levels = c(0, 1),
  labels = c("not holiday", "holiday")
))
hr$weekday <- with(hr, factor(weekday,
  levels = c(0:6),
  labels = c("sun", "mon", "tue", "wed", "thu", "fri", "sat")
))
hr$workingday <- with(hr, factor(workingday,
  levels = c(0, 1),
  labels = c("not working day", "working day")
))
hr$weathersit <- with(hr, factor(weathersit,
  levels = c(1, 2, 3, 4),
  labels = c("clear", "cloudy", "light rain/fog", "heavy rain/fog")
))
# hr$hr=with(hr,factor(hr,levels = c(0:23),
#                      labels = c("0","1","2","3","4","5","6","7","8","9","10","11",
#                                 "12","13","14","15","16","17","18","19","20","21","22","23"),ordered = TRUE))

hr$day <- as.integer(substr(hr$dteday, 9, 10))

# factors to binary
weathersit <- sapply(levels(hr$weathersit), function(x) as.factor(x == hr$weathersit))
colnames(weathersit)[3:4] <- c("lightrain", "heavyrain")
holiday <- sapply(levels(hr$holiday), function(x) as.factor(x == hr$holiday))
colnames(holiday) <- c("notholiday", "holidayc")
weekday <- sapply(levels(hr$weekday), function(x) as.factor(x == hr$weekday))
##
hr <- cbind(hr, weathersit, holiday, weekday)
####### Data for a specific month
hr11 <- subset(hr, yr == "2012" & mnth == "jan")
##
test <- subset(hr, yr == "2011" & mnth == "jan")
#
head(hr11)
head(test)
#
###### model for registered and casual users
m2 <- as.formula(casual ~ day + hr + holiday + weekday + weathersit +
  atemp + hum + windspeed + weekday)
######
####### glm models
cmp.reg <- gam.cmp(m2, family = cmp, data = hr11)
pos.reg <- glm(m2, family = poisson(), data = hr11)

### Table A2
require(stargazer)
summary.gam.cmp(cmp.reg)
stargazer(pos.reg)

# mob model for comparision
## input model formualtion
formula <- as.formula(casual ~ atemp + hum + hr + day)
fformula <- NULL # as.formula(~s(hr)+s(day))
vcformula <- as.formula(~ windspeed + clear + cloudy + lightrain + heavyrain + notholiday + holidayc + sun + mon + tue + thu + fri + sat)

##### Boosting
### Training for the terminal nodes
grid.M <- c(5, 10, 15, 20, 25, 30)

# grid.M1 <- c(45,50,55,60,65,70,75,80)
tdev <- rep(0, length(grid.M))
terr <- rep(0, length(grid.M))
tmin <- rep(0, length(grid.M))
niter <- rep(0, length(grid.M))
tdev1 <- tdev
terr1 <- terr
tmin1 <- tmin
niter1 <- niter
for (i in 1:length(grid.M))
{
  tres <- Boost.Tree(formula = formula, fformula = fformula, vcformula = vcformula, data = hr11, eta = 0.8, mini.size = 20, M = grid.M[i], nBoost = 2000)
  tdev[i] <- tres$deviance[tres$iter]
  terr[i] <- tres$error.sq[which.min(tres$error.sq[1:tres$iter])]
  tmin[i] <- which.min(tres$error.sq[1:tres$iter])
  niter[i] <- tres$iter
  #
  tres1 <- Boost.Tree(formula = formula, fformula = fformula, vcformula = vcformula, data = test, eta = 0.8, mini.size = 20, M = grid.M[i], nBoost = tres$iter)
  tdev1[i] <- tres1$deviance[tres1$iter]
  terr1[i] <- tres1$error.sq[which.min(tres1$error.sq[1:tres1$iter])]
  tmin1[i] <- which.min(tres1$error.sq[1:tres1$iter])
  niter1[i] <- tres1$iter
} ## 0.05 for nu function


#############################################################

pdf("gb-train1-bike.pdf", 7, 4)
#
par(mfrow = c(1, 2))
plot(grid.M, tdev, xlab = "M", ylab = "-2l", pch = 1, type = "l", lwd = 2, cex.axis = 1.5, cex.lab = 1.5, ylim = c(1500, 3500))
points(grid.M, tdev1, type = "l", lwd = 2, lty = 2)
#
plot(grid.M, terr, xlab = "M", ylab = "Prediction Error", pch = 1, type = "l", lwd = 2, cex.axis = 1.5, cex.lab = 1.5, ylim = c(1000, 14000))
points(grid.M, terr1, type = "l", lwd = 2, lty = 2)
dev.off()


############## CMPBoost model ###################################
cmp.bt1 <- Boost.Tree(formula = formula, fformula = fformula, vcformula = vcformula, data = hr11, eta = 0.8, M = 25, nBoost = 2000)

## variable importance plot: Figure 9

pdf("varimp-bike.pdf", 9, 6)
par(mar = c(8, 3, 0.5, 0.5))
barplot(cmp.bt1$variable.imp.lam / sum(cmp.bt1$variable.imp.lam), names.arg = names(cmp.bt1$varPart1), las = 2, cex.names = 1.5)
dev.off()
### partial plot
vnames <- c("windspeed", "lightrain", "notholiday", "sat", "sun", "mon", "cloudy")
pat.bt1 <- Partial.Boost(cmp.bt1, vnames)

## partial plots for CMPBoost : Figure 10
# require(KernSmooth)
pdf("part1-bike.pdf", 9, 3)

par(mfrow = c(1, 4))
par(mar = c(5, 5, 0.5, 0.5))
plot(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 2], pch = 19, xlab = "windspeed", ylab = expression(beta[1](windspeed)), cex.lab = 1.5)
points(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 2], type = "l", lty = 2)
# fit <- locpoly(pat.bt1$ulevel.lam[,1][1:22],pat.bt1$coefs.lam[[1]][,2], bandwidth = 0.1)
# lines(fit)
par(mar = c(5, 5, 0.5, 0.5))
plot(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 3], pch = 19, xlab = "windspeed", ylab = expression(beta[2](windspeed)), cex.lab = 1.5)
points(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 3], type = "l", lty = 2)
#
par(mar = c(5, 5, 0.5, 0.5))
plot(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 4], pch = 19, xlab = "windspeed", ylab = expression(beta[3](windspeed)), cex.lab = 1.5)
points(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 4], type = "l", lty = 2)
#
par(mar = c(5, 5, 0.5, 0.5))
plot(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 5], pch = 19, xlab = "windspeed", ylab = expression(beta[4](windspeed)), cex.lab = 1.5)
points(pat.bt1$ulevel.lam[, 1][1:22], pat.bt1$coefs.lam[[1]][, 5], type = "l", lty = 2)
dev.off()

#######################################################################
#################### CMPMOB Trees ####################################
#### Exhaustive Search

cmp.mob1 <- cmpmob(formula, fformula, vcformula, data = hr11, family = cmp, sflag = TRUE)
###  10% change point Search
cmp.mob11 <- cmpmob(formula, fformula, vcformula, data = hr11, family = cmp, sflag = FALSE)


##############################################################
############### Plotting CMPMOB Tree #########################

pdf("cmp-mobtree.pdf", 14, 14)
par(mfrow = c(1, 1))
par(cex = 0.4)
par(mar = c(0.5, 0.5, 0.5, 0.5))
plot(cmp.mob1$tree)
dev.off()


#
node1 <- nodeids(cmp.mob1$tree, terminal = TRUE)
ldev1 <- sum(unlist(nodeapply(cmp.mob1$tree, ids = node1, FUN = function(n) info_node(n)$deviance)))
gdev1 <- cmp.mob1$glob$deviance
#

#################################################################
############### Plotting  Poisson MOB Tree #########################

pos.mob <- glmtree(casual ~ atemp + hum + hr + day | windspeed + clear + cloudy + lightrain + heavyrain + notholiday + holidayc + sun + mon + tue + thu + fri + sat, data = hr11, family = poisson())
pdf("pos-mob.pdf", 11, 7)
plot.modelparty(pos.mob)
dev.off()

#
node2 <- nodeids(pos.mob, terminal = TRUE)
ldev2 <- sum(unlist(nodeapply(pos.mob, ids = node2, FUN = function(n) info_node(n)$aic)))
# ## test mooth
# f <- as.formula(casual~s(windspeed,by=weekday)+s(windspeed,by=holiday)+s(hr)+s(day)+s(hum)+s(atemp))
# testgam <- cmp::gam.cmp(f, data=hr11)
#
####

##################################################################################################
##################### Generalized MOB with fixed effect terms ######################################

###  Input model formulation
formula <- list(as.formula(casual ~ atemp), as.formula(casual ~ windspeed)) #+hr+day
fformula <- list(as.formula(~ s(day) + s(hr)), as.formula(~hum))
vcformula <- list(as.formula(~ clear + cloudy + lightrain + heavyrain + notholiday + holidayc + sun + mon + tue + thu + fri + sat), as.formula(~ clear + cloudy + lightrain + heavyrain + notholiday + holidayc + sun + mon + tue + thu + fri + sat))

### CMPMOB Tree
cmp.mob2 <- cmpmob(formula, fformula, vcformula, data = hr11, family = cmp, sflag = TRUE)
### vc model with change point method

pdf("cmp-mob-gen.pdf", 11, 7)
plot(cmp.mob2$tree)
dev.off()

pdf("cmp-mob-glob.pdf", 7, 4)
par(mfrow = c(1, 2))
plot.gam(cmp.mob2$glob, rug = TRUE)
dev.off()


node2 <- nodeids(cmp.mob2$tree, terminal = TRUE)
nterm2 <- length(node2)
ldev2 <- sum(unlist(nodeapply(cmp.mob2$tree, ids = node2, FUN = function(n) info_node(n)$deviance)))
gdev2 <- cmp.mob2$glob$deviance



##############################################################################
############### CMPBoost Model ###################################################

cmp.bt2 <- Boost.Tree(formula = formula, fformula = fformula, vcformula = vcformula, data = hr11, eta = 0.8, M = 25, nBoost = 2000)


###############################################################################
############# variable importance plot ########################################

pdf("varimp-bike2.pdf", 9, 6)
par(mfrow = c(1, 2))
par(mar = c(8, 3, 0.5, 0.5))
barplot(cmp.bt2$variable.imp.lam / sum(cmp.bt2$variable.imp.lam), names.arg = names(cmp.bt2$varPart1), las = 2, cex.names = 1.5)
barplot(cmp.bt2$variable.imp.nu / sum(cmp.bt2$variable.imp.nu), names.arg = names(cmp.bt2$varPart2), las = 2, cex.names = 1.5)
dev.off()

###############################################################################
################## partial plot ###############################################

vnames <- c("sat", "fri")
pat.bt2 <- Partial.Boost(cmp.bt2, vnames)

pat.mb2 <- Partial.mob(cmp.mob2, vnames)

## partial plots for CMPBoost
# require(KernSmooth)
pdf("part2-bike.pdf", 9, 4)
par(mfrow = c(2, 4))

### for lam
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.bt2$ulevel.lam[, 2][1:2]), pat.bt2$coefs.lam[[2]][, 1], xlab = "", ylab = expression(beta[0](fri)), cex.lab = 1.5, main = "CMPBoost", pch = 19)
points(pat.bt2$ulevel.lam[, 2][1:2], pat.bt2$coefs.lam[[2]][, 1], type = "l", lty = 2)
# fit <- locpoly(pat.bt1$ulevel.lam[,1][1:22],pat.bt1$coefs.lam[[1]][,2], bandwidth = 0.1)
#
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.mb2$ulevel.lam[, 2][1:2]), pat.mb2$coefs.lam[[2]][, 1], pch = 19, xlab = "", ylab = expression(beta[0](fri)), cex.lab = 1.5, main = "CMPMOB")
points(pat.mb2$ulevel.lam[, 2][1:2], pat.mb2$coefs.lam[[2]][, 1], type = "l", lty = 2)

# lines(fit)
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.bt2$ulevel.lam[, 2][1:2]), pat.bt2$coefs.lam[[2]][, 2], pch = 19, xlab = "", ylab = expression(beta[1](fri)), cex.lab = 1.5, main = "CMPBoost")
points(pat.bt2$ulevel.lam[, 2][1:2], pat.bt2$coefs.lam[[2]][, 2], type = "l", lty = 2)

#
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.mb2$ulevel.lam[, 2][1:2]), pat.mb2$coefs.lam[[2]][, 2], pch = 19, xlab = "", ylab = expression(beta[1](fri)), cex.lab = 1.5, main = "CMPMOB")
points(pat.mb2$ulevel.lam[, 2][1:2], pat.mb2$coefs.lam[[2]][, 2], type = "l", lty = 2)
###############
####### for nu
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.bt2$ulevel.nu[, 1][1:2]), pat.bt2$coefs.nu[[1]][, 1], pch = 19, xlab = "", ylab = expression(gamma[0](sat)), cex.lab = 1.5, main = "CMPBoost")
points(pat.bt2$ulevel.nu[, 1][1:2], pat.bt2$coefs.nu[[1]][, 1], type = "l", lty = 2)
# fit <- locpoly(pat.bt1$ulevel.lam[,1][1:22],pat.bt1$coefs.lam[[1]][,2], bandwidth = 0.1)
#
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.mb2$ulevel.lam[, 1][1:2]), pat.mb2$coefs.nu[[1]][, 1], pch = 19, xlab = "", ylab = expression(gamma[0](sat)), cex.lab = 1.5, main = "CMPMOB")
points(pat.mb2$ulevel.lam[, 1][1:2], pat.mb2$coefs.nu[[1]][, 1], type = "l", lty = 2)

# lines(fit)
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.bt2$ulevel.nu[, 1][1:2]), pat.bt2$coefs.nu[[1]][, 2], pch = 19, xlab = "", ylab = expression(gamma[1](sat)), cex.lab = 1.5, main = "CMPBoost")
points(pat.bt2$ulevel.nu[, 1][1:2], pat.bt2$coefs.nu[[1]][, 2], type = "l", lty = 2)

#
par(mar = c(2, 5, 1.5, 0.5))
plot(as.factor(pat.mb2$ulevel.lam[, 1][1:2]), pat.mb2$coefs.nu[[1]][, 2], pch = 19, xlab = "", ylab = expression(gamma[1](sat)), cex.lab = 1.5, main = "CMPMOB")
points(pat.mb2$ulevel.lam[, 1][1:2], pat.mb2$coefs.nu[[1]][, 2], type = "l", lty = 2)
#
dev.off()
